-- Shading value
if use_shading == nil then
	use_shading = true
end
RayCast3D.useShading(use_shading)

-- Texture quality value
if norm_acc == nil then
	norm_acc = 3
end

-- Music related values
if isSFX == nil then
	isSFX = true
end
if isMusic == nil then
	isMusic = true
end

-- Format a number of milliseconds in a time-like string (Example: 123000 milliseconds = 02:03)
function FormatTime(milliseconds)
	seconds = math.floor(milliseconds/1000)
	minute = math.floor(seconds/60)
	seconds = seconds%60
	hours = math.floor(minute/60)
	minute = minute%60
	if minute < 10 then
		minute = "0"..minute
	end
	if seconds < 10 then
		seconds = "0"..seconds
	end
	if hours < 10 then
		hours = "0"..hours
	end
	return hours..":"..minute..":"..seconds
end

-- Changes current shading effect setting
function changeShadingValue()
	use_shading = not use_shading
	RayCast3D.useShading(use_shading)
	if use_shading then
		graphics_voices[2] = "Shading: Enabled"
	else
		graphics_voices[2] = "Shading: Disabled"
	end
end

-- Changes current texture quality setting
function changeTextureQuality()
	norm_acc = norm_acc + 1
	if norm_acc > #accs then
		norm_acc = 1
	end
	graphics_voices[1] = "Textures Quality : "..accs[norm_acc]
	RayCast3D.setAccuracy(norm_acc)
end

-- Plays a BGM
function PlayMusic(file, loop)
	Sound.play(file, loop)
	if not isMusic then
		Sound.pause(file)
	end
end

-- Plays a sound effect
function PlaySFX(file, loop)
	if isSFX then
		Sound.play(file, loop)
	end
end

-- Changes current BGM setting
function changeMusicValue()
	isMusic = not isMusic
	if isMusic then
		audio_voices[1] = "BGM Music: On"
		Sound.resume(bgm)
	else
		audio_voices[1] = "BGM Music: Off"
		Sound.pause(bgm)
	end
end

-- Changes current SFX setting
function changeSoundValue()
	isSFX = not isSFX
	if isSFX then
		audio_voices[2] = "Sounds Effects: On"
	else
		audio_voices[2] = "Sounds Effects: Off"
	end
end

-- Generic save/load game menu drawer with callback
local black = Color.new(0,0,0)
function DrawSavegameMenu(title, new_voice)
	j = 1
	Graphics.fillRect(500,902,20,248,black)
	if new_voice then
		if save_idx == 1 then
			Font.setPixelSizes(ttf,28)
			Font.print(ttf, x, 80, "[New Save]", yellow)
			j = j + 1
		end
		save_idx = save_idx - 1
	end
	Font.setPixelSizes(ttf,40)
	Font.print(ttf, 8, 15, title, cyan)
	Font.setPixelSizes(ttf,28)
	for i, save in pairs(save_slots) do
		if i == save_idx then
			x = 20
			Font.print(ttf, x, 50 + 30 * j, save.name, yellow)
			j=j+1
			Graphics.drawScaleImage(501,21,save.image,0.416,0.416)
			Font.print(ttf, 500, 260, "Seed: " .. save.seed, white)
			Font.print(ttf, 500, 290, "Scrolls: " .. save.scrolls, white)
			Font.print(ttf, 500, 320, "Map Size: " .. save.size .. "x" .. save.size, white)		
			Font.print(ttf, 500, 350, "Playtime: " .. FormatTime(save.playtime), white)
		elseif i > save_idx then
			x = 15
			Font.print(ttf, x, 50 + 30 * j, save.name, white)
			j=j+1
		end
		if j > 28 then
			break
		end
	end
	if new_voice then
		save_idx = save_idx + 1
	end
end

-- Generic confirmation request function
function askPlayer(text, anim_idx)
	local ask_i = 1
	oldpad = SCE_CTRL_CROSS
	while true do
		local ask_pad = Controls.read()
		Graphics.initBlend()
		if use_shading then
			Graphics.drawImage(0, 0, skybox)
		else
			Graphics.fillRect(0,960,0,544,ceil_c) -- Simulates skybox; currently faster then sky rendering
			Graphics.fillRect(0,960,273,544,floor_c) -- Simulates floor; currently faster then floor rendering
		end
		RayCast3D.renderScene(0,0)
		if anim_idx ~= nil then Graphics.drawPartialImage(284,440,0,104 * anim_idx,376,104,anim) end
		Font.setPixelSizes(ttf,28)
		Font.print(ttf, 15, 15, text, cyan)
		if ask_i == 1 then
			Font.print(ttf, 20, 80, "Yes", yellow)
			Font.print(ttf, 20, 110, "No", white)
			if Controls.check(ask_pad, SCE_CTRL_CROSS) and not Controls.check(oldpad, SCE_CTRL_CROSS) then
				PlaySFX(step, NO_LOOP)
				oldpad = SCE_CTRL_CROSS
				return true
			elseif Controls.check(ask_pad, SCE_CTRL_DOWN) and not Controls.check(oldpad, SCE_CTRL_DOWN) then
				PlaySFX(step, NO_LOOP)
				ask_i = 2
			end
		else
			Font.print(ttf, 20, 80, "Yes", white)
			Font.print(ttf, 20, 110, "No", yellow)
			if Controls.check(ask_pad, SCE_CTRL_CROSS) and not Controls.check(oldpad, SCE_CTRL_CROSS) then
				PlaySFX(step, NO_LOOP)
				oldpad = SCE_CTRL_CROSS
				return false
			elseif Controls.check(ask_pad, SCE_CTRL_UP) and not Controls.check(oldpad, SCE_CTRL_UP) then
				PlaySFX(step, NO_LOOP)
				ask_i = i
			end
		end
		Graphics.termBlend()
		Screen.flip()
		oldpad = ask_pad
	end
end

-- CRC32 Implementation
local crc_tbl = {}
for idx=0, 255 do
	local val = idx
	for _=1, 8 do
		if val % 2 == 1 then
			val = bit32.bxor(0xEDB88320,bit32.rshift(val,1))
		else
			val = bit32.rshift(val,1)
		end
	end
	crc_tbl[idx] = val
end
function crc32(buffer)
	local hash = 0xFFFFFFFF
	local len = string.len(buffer)
	for idx=1, len do
		hash = bit32.bxor((crc_tbl[bit32.band(bit32.bxor(hash,string.byte(buffer, idx)),255)]),bit32.rshift(hash,8))
	end	
	return bit32.bxor(hash,0xFFFFFFFF)	
end

-- Converts an integer to a string
function int2str(num, size)
	local tmp = 0
	local bytes = ""
	local extra_shift = 4 - size
	while tmp < size do
		tmp = tmp + 1
		bytes = string.char(bit32.rshift(bit32.lshift(num,(bit32.lshift(extra_shift+tmp-1,3))),(bit32.lshift(extra_shift+size-1,3)))) .. bytes
	end
	return bytes
end

-- Converts a string to an integer
function str2int(str)
	size = string.len(str)
	local b1,b2,b3,b4 = string.byte(str,1,size)
	if size == 1 then
		return b1
	elseif size == 2 then
		return b1 + b2*0x100
	else
		return b1 + b2*0x100 + b3*0x10000 + b4*0x1000000
	end
end
