-- Loading RayCast3D Engine
dofile("app0:/files/raycast3d.lua")

-- Create savegames folder if doesn't exist
SAV_FOLDER = "ux0:/data/Labyrinth 3D"
System.createDirectory(SAV_FOLDER)

-- Loading config file, if it exists
if System.doesFileExist(SAV_FOLDER.."/config.cfg") then
	dofile(SAV_FOLDER.."/config.cfg")
end
local config_update = false

-- Initializing everything
dofile("app0:/files/primm.lua") -- Primm's Algorithm
dofile("app0:/files/funcs.lua") -- Generic Functions
Sound.init() -- Sound Module
ttf = Font.load("app0:/files/main.ttf") -- Font
cfg_icon = Graphics.loadImage("app0:/files/icon.png")

-- Map Details
wall_height = 64
tile_size = 64

-- Player Speed
pl_speed = 5
cam_speed = 50

-- Showing intro
skip_logo = false
if Controls.check(Controls.read(), SCE_CTRL_LTRIGGER) then
	skip_logo = true
end
if not skip_logo then
	lpp = Graphics.loadImage("app0:/files/lpp.png")
	rc = Graphics.loadImage("app0:/files/rc.png")
	rcs = Sound.open("app0:/files/rc.ogg")
	lpps = Sound.open("app0:/files/lpp.ogg")
	ltmr = Timer.new()
	a = 0
	while true do
		tmppad = Controls.read()
		if a > 255 then
			a = 255
			if stp == nil then
				stp = true
				Timer.reset(ltmr)
			end
		elseif stp == nil then
			a = math.floor(Timer.getTime(ltmr) / 10)
		else
			if stp then
				if plyd == nil then
					Sound.play(lpps, NO_LOOP)
					plyd = true
				end
				if Timer.getTime(ltmr) > 2000 then
					stp = false
					Timer.reset(ltmr)
				end
			elseif a > 0 then
				a = 255 - math.floor(Timer.getTime(ltmr) / 10)
			else
				break
			end
		end
		Graphics.initBlend()
		Graphics.fillRect(0,960,0,544,Color.new(255,255,255))
		Graphics.drawImage(330,172,lpp,Color.new(255,255,255,math.min(a, 255)))
		Graphics.termBlend()
		Screen.flip()
		if Controls.check(tmppad, SCE_CTRL_START) then
			Sound.pause(lpps)
			break
		end
	end
	Timer.reset(ltmr)
	a = 0
	stp = nil
	plyd = nil
	while true do
		tmppad2 = Controls.read()
		if a > 255 then
			a = 255
			if stp == nil then
				stp = true
				Timer.reset(ltmr)
			end
		elseif stp == nil then
			a = math.floor(Timer.getTime(ltmr) / 10)
		else
			if stp then
				if plyd == nil then
					Sound.play(rcs, NO_LOOP)
					plyd = true
				end
				if Timer.getTime(ltmr) > 2000 then
					stp = false
					Timer.reset(ltmr)
				end
			elseif a > 0 then
				a = 255 - math.floor(Timer.getTime(ltmr) / 10)
			else
				break
			end
		end
		Graphics.initBlend()
		Graphics.fillRect(0,960,0,544,Color.new(0,0,0))
		Graphics.drawPartialImage(330, 152, rc, 150, 10, 360, 240, Color.new(255,255,255,math.min(a, 255)))
		Graphics.termBlend()
		Screen.flip()
		if Controls.check(tmppad2, SCE_CTRL_START) and not Controls.check(tmppad, SCE_CTRL_START) then
			Sound.pause(rcs)
			break
		end
		tmppad = tmppad2
	end
	Sound.close(rcs)
	Sound.close(lpps)
	Graphics.freeImage(lpp)
	Graphics.freeImage(rc)
	Timer.destroy(ltmr)
end

Graphics.initBlend()
Screen.clear()
Font.setPixelSizes(ttf, 40)
Font.print(ttf, 290,252, "Loading, please wait...", Color.new(255,255,255))
Font.setPixelSizes(ttf, 28)
Graphics.drawImage(550, 455, cfg_icon)
Font.print(ttf, 90,452, "Don't close the game when the icon       appears on the screen.", Color.new(255,255,255))
Graphics.termBlend()
Screen.flip()
Graphics.initBlend()
Screen.clear()
Font.setPixelSizes(ttf, 40)
Font.print(ttf, 290,252, "Loading, please wait...", Color.new(255,255,255))
Font.setPixelSizes(ttf, 28)
Graphics.drawImage(550, 455, cfg_icon)
Font.print(ttf, 90,452, "Don't close the game when the icon       appears on the screen.", Color.new(255,255,255))
Graphics.termBlend()
Screen.flip()
Graphics.initBlend()
Screen.clear()
Font.setPixelSizes(ttf, 40)
Font.print(ttf, 290,252, "Loading, please wait...", Color.new(255,255,255))
Font.setPixelSizes(ttf, 28)
Graphics.drawImage(550, 455, cfg_icon)
Font.print(ttf, 90,452, "Don't close the game when the icon       appears on the screen.", Color.new(255,255,255))
Graphics.termBlend()
Screen.flip()

-- Caching savegames
save_slots = {}
savegames = System.listDirectory(SAV_FOLDER)
for i, file in pairs(savegames) do
	if (not file.directory) and (string.upper(string.sub(file.name,-4)) == ".SAV") then
		local sav_hdl = System.openFile(SAV_FOLDER .. "/" .. file.name, FREAD)
		local sav_size = io.size(sav_hdl)
		if sav_size > 0x100 then
			local exp_checksum = str2int(System.readFile(sav_hdl, 4))
			local sav_buffer = System.readFile(sav_hdl, sav_size)
			System.closeFile(sav_hdl)
			local sav_checksum = crc32(sav_buffer)
			if exp_checksum == sav_checksum then
				local seed = str2int(string.sub(sav_buffer,1,4))
				local map_size = str2int(string.sub(sav_buffer,5,6))
				local playtime = str2int(string.sub(sav_buffer,7,10))
				local used_scrolls = str2int(string.sub(sav_buffer,11,12))
				local x = str2int(string.sub(sav_buffer,13,16))
				local y = str2int(string.sub(sav_buffer,17,20))
				local angle = str2int(string.sub(sav_buffer,21,24))
				local data = str2int(string.sub(sav_buffer,25,26))
				local scrolls = data-used_scrolls
				local offs = 27
				local marked_walls_data = ""
				if scrolls >= 0 then
					if used_scrolls > 0 then
						offs = offs + bit32.lshift(used_scrolls,3)
						marked_walls_data = string.sub(sav_buffer,27,offs-1)
					end	
				end
				local sshot_data = string.sub(sav_buffer,offs)
				local t_hdl = System.openFile(SAV_FOLDER .. "/labyr.inth",FCREATE)
				System.writeFile(t_hdl, sshot_data, string.len(sshot_data))
				System.closeFile(t_hdl)
				local sshot = Graphics.loadImage(SAV_FOLDER .. "/labyr.inth")
				System.deleteFile(SAV_FOLDER .. "/labyr.inth")
				table.insert(save_slots, {["size"] = map_size, ["name"] = file.name, ["playtime"] = playtime, ["seed"] = seed, ["x"] = x, ["y"] = y, ["angle"] = angle, ["scrolls"] = scrolls, ["marked"] = marked_walls_data, ["image"] = sshot})
			end
		else
			System.closeFile(sav_hdl)
		end
	end
end

-- Loading textures
W = Graphics.loadImage("app0:/files/wall.png")
M = Graphics.loadImage("app0:/files/marked.png")
anim = Graphics.loadImage("app0:/files/player.png")
logo = Graphics.loadImage("app0:/files/logo.png")
skybox = Graphics.loadImage("app0:/files/skybox.png")

-- Loading sounds
walk = Sound.open("app0:/files/footsteps.ogg")
step = Sound.open("app0:/files/step.ogg")
bgm = Sound.open("app0:/files/menu.ogg")
Sound.play(bgm, LOOP)
Sound.play(walk, LOOP)
Sound.pause(walk)

-- Main Menu Map Details
dmap = {
	W,W,W,W,W,W,
	W,0,0,0,W,W,
	W,0,W,0,0,W,
	W,0,W,W,0,W,
	W,0,0,W,0,W,
	W,W,0,0,0,W,
	W,W,W,W,W,W
}
dmap_width = 6
dmap_height = 7
state = 0

-- Setting up RayCast3D Engine
RayCast3D.setResolution(960, 544)
RayCast3D.setViewsize(60)
RayCast3D.loadMap(dmap, dmap_width, dmap_height, tile_size, wall_height)
RayCast3D.spawnPlayer(96, 96, 90)

-- Set CPU speed & texture quality
System.setCpuSpeed(444)
RayCast3D.setAccuracy(norm_acc)

-- Menu Stuffs
menu_title = "MAIN MENU"
credits_voices = {
	"yDevelopers:",
	"Rinnegatamante",
	" ","yImages:",
	"Misledz",
	" ","ySprites:",
	"ETTiNGRiNDER",	
	" ","yMusics:",
	"aliaspharow",
	" ","ySounds:",
	"soundjay.com",
	" ","yVoices:",
	"oddcast.com",
	" ","yTesters:",
	"Rinnegatamante", "ihaveamac", "gnmmarechal",
	" ","yPatreon Supporters:",
	"XandridFire","Styde Pregny","Billy McLaughlin II",
	" ","yEngines:",
	"Lua Player Plus Vita", "RayCast3D Engine"
}
sizes = {"Smallest", "Small", "Medium", "Big", "Huge", "Titanic", "Monstrous"}
accs = {"Best", "Very High", "High", "Medium", "Low", "Very Low", "Worst"}
main_menu_voices = {"New Game", "Load Game", "Graphics Settings", "Audio Settings", "Credits", "Exit Game"}
graphics_voices = {"Textures Quality: " .. accs[norm_acc], "Shading: Enabled", "Return Main Menu"}
audio_voices = {"BGM Musics : On", "Sounds Effects : On", "Return Main Menu"}
new_game_voices = {"Map Size : Small", "Start a new game", "Start a new game using a seed", "Return Main Menu"}
menus_voices = {{}, main_menu_voices, new_game_voices, graphics_voices, audio_voices}
tsizes = {15, 25, 40, 60, 80, 100, 150}
scrolls_num = {0, 0, 1, 2, 4, 6, 8}
idx_voices = 1
idx_sizes = 2
gyrosens = 1
version = "v.1.4"
menu_idx = 1 -- 1 = Logo , 2 = Main Menu, 3 = New Game, 4 = Graphics Settings, 5 = Audio Settings
load_idx = 1

-- Set variables
credits = false
seed_game = false
load_screen = false
gyromode = false

-- Setting colors
ceil_c = Color.new(83, 69, 59, 255)
floor_c = Color.new(25, 17, 15, 255)
wall_c = Color.new(25, 17, 15, 0)
white = Color.new(255,255,255)
cyan = Color.new(0,255,255)
yellow = Color.new(255,255,0)
gray = Color.new(192,192,192)
RayCast3D.enableFloor(false)
RayCast3D.setFloorColor(Color.new(29,14,7))
RayCast3D.setWallColor(wall_c)

-- Set shading depth
RayCast3D.setDepth(300)

-- Drawing menu
function DrawMenu(title)
	i = 1
	Font.setPixelSizes(ttf,40)
	Font.print(ttf, 8, 15, title, cyan)
	Font.setPixelSizes(ttf,28)
	while i <= #menus_voices[menu_idx] do
		if i == idx_voices then
			color = yellow
			x = 20
		else
			color = white
			x = 15
		end
		if menu_idx == 2 then
			if i == 2 and #save_slots == 0 then
				color = gray
			end
		end
		Font.print(ttf, x, 50 + 30 * i, menus_voices[menu_idx][i], color)
		i = i + 1
	end
end

-- Drawing credits
function DrawCredits()
	tm = Timer.getTime(tmr)
	for i, voice in pairs(credits_voices) do
		if string.sub(voice,1,1) == "y" then
			v = string.sub(voice,2,-1)
			color = yellow
		else
			v = voice
			color = white
		end
		y = 540 + 30 * i - math.floor(tm / 15)
		if y >= 0 and y <= 600 then
			Font.print(ttf, 15, y, v, color)
		end
	end
	if tm >= 23000 or Controls.check(Controls.read(), SCE_CTRL_START) then
		credits = false
	end
end

-- Seed Keyboard
local seed_val = nil
function InitSeedKeyboard()
	Keyboard.show("Insert seed", "0", 16, TYPE_NUMBER, MODE_TEXT)
	seed_val = nil
end
function DrawSeedKeyboard()
	status = Keyboard.getState()
	if status ~= RUNNING then
		if status ~= CANCELED then
			seed_val = Keyboard.getInput()
		else
			seed_game = false
		end
		Keyboard.clear()		
	end
end

-- HIDDEN ZONE (SSSSHHH!)
-- Hidden Easter Egg (Why not?)
easter_egg = false
set_egg = false
konami_code = {SCE_CTRL_UP, SCE_CTRL_UP, SCE_CTRL_DOWN, SCE_CTRL_DOWN, SCE_CTRL_LEFT, SCE_CTRL_RIGHT, SCE_CTRL_LEFT, SCE_CTRL_RIGHT}
konami_idx = 1
konami_tmr = Timer.new()
-- Cheats (No ActionReplay needed!)
cheat1 = {SCE_CTRL_CROSS, SCE_CTRL_UP, SCE_CTRL_CROSS, SCE_CTRL_DOWN, SCE_CTRL_CIRCLE, SCE_CTRL_LEFT, SCE_CTRL_CIRCLE, SCE_CTRL_RIGHT} -- x99 Scrolls
cheat2 = {SCE_CTRL_UP, SCE_CTRL_DOWN, SCE_CTRL_LEFT, SCE_CTRL_RIGHT, SCE_CTRL_UP, SCE_CTRL_DOWN, SCE_CTRL_LEFT, SCE_CTRL_RIGHT, SCE_CTRL_CIRCLE, SCE_CTRL_CROSS} -- NoClip Mode
cheat1_idx = 1
cheat2_idx = 1
noclip = false

-- Game Timer
tmr = Timer.new()

sec = Timer.new()
frame = 0
fps = 0
while true do
	
	-- Updating config file if requested
	if config_update then
		config_update = false
		if System.doesFileExist(SAV_FOLDER.."/config.cfg") then
			System.deleteFile(SAV_FOLDER.."/config.cfg")
		end
		local cfg_hdl = System.openFile(SAV_FOLDER.."/config.cfg", FCREATE)
		local buffer = "use_shading="
		if use_shading then
			buffer = buffer .. "true"
		else
			buffer = buffer .. "false"
		end
		buffer = buffer .. "\nnorm_acc=" .. norm_acc .. "\nisSFX="
		if isSFX then
			buffer = buffer .. "true\nisMusic="
		else
			buffer = buffer .. "false\nisMusic="
		end
		if isMusic then
			buffer = buffer .. "true"
		else
			buffer = buffer .. "false"
		end
		System.writeFile(cfg_hdl, buffer, string.len(buffer))
		System.closeFile(cfg_hdl)
		update_tmr = Timer.new()
		cfg_angle = 1.0
	end
	
	if menu_idx == 2 then
		menu_title = "MAIN MENU"
	end
	
	-- Rendering scene
	Graphics.initBlend()
	if use_shading then
		Graphics.drawImage(0, 0, skybox)
	else
		Graphics.fillRect(0,960,0,544,ceil_c) -- Simulates skybox; currently faster then sky rendering
		Graphics.fillRect(0,960,273,544,floor_c) -- Simulates floor; currently faster then floor rendering
	end
	RayCast3D.renderScene(0,0)
	if menu_idx == 1 then
		Graphics.drawImage(320,152,logo)
	end
	
	-- Rendering config icon
	if update_tmr ~= nil then
		if Timer.getTime(update_tmr) > 1500 then
			Timer.destroy(update_tmr)
			update_tmr = nil
		end
		Graphics.drawRotateImage(25, 520, cfg_icon, cfg_angle)
		cfg_angle = cfg_angle + 1
	end
	
	-- Background animation
	if not easter_egg and not set_egg then
		if state == 0 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.y >= 272 then
				state = 1
			end
		elseif state == 1 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle <= 35 then
				state = 2
			end
		elseif state == 2 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.y >= 338 then
				state = 3
			end
		elseif state == 3 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			RayCast3D.movePlayer(FORWARD, pl_speed)
			RayCast3D.movePlayer(RIGHT, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle >= 200 and cam.angle <= 320 then
				state = 4
			end
		elseif state == 4 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.y <= 320 then
				state = 5
			end
		elseif state == 5 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle < 270 then
				state = 6
			end
		elseif state == 6 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.y <= 200 then
				state = 7
			end
		elseif state == 7 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle < 230 then
				state = 8
			end
		elseif state == 8 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.y <= 100 then
				state = 9
			end
		elseif state == 9 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle <= 180 then
				state = 10
			end
		elseif state == 10 then
			RayCast3D.movePlayer(FORWARD, pl_speed)
			cam = RayCast3D.getPlayer()
			if cam.x <= 100 then
				state = 11
			end
		elseif state == 11 then
			RayCast3D.rotateCamera(LEFT, cam_speed)
			cam = RayCast3D.getPlayer()
			if cam.angle <= 90 then
				state = 0
			end
		end
	end
	
	-- Main Menu
	pad = Controls.read()
	if menu_idx == 1 then
		if Controls.check(pad, cheat1[cheat1_idx]) and not Controls.check(oldpad, cheat1[cheat1_idx]) then
			cheat1_idx = cheat1_idx + 1
			Timer.reset(konami_tmr)
			if cheat1_idx > #cheat1 then
				scrolls_num = {99, 99, 99, 99, 99, 99, 99}
				Timer.reset(konami_tmr)
				PlaySFX(step, NO_LOOP)
				cheat1_idx = 1
			end
		end
		if Controls.check(pad, cheat2[cheat2_idx]) and not Controls.check(oldpad, cheat2[cheat2_idx]) then
			cheat2_idx = cheat2_idx + 1
			Timer.reset(konami_tmr)
			if cheat2_idx > #cheat2 then
				PlaySFX(step, NO_LOOP)
				noclip = true
				Timer.reset(konami_tmr)
				cheat2_idx = 1
			end
		end
		if Timer.getTime(konami_tmr) > 800 then
			cheat1_idx = 1
			cheat2_idx = 1
			Timer.reset(konami_tmr)
		end
		if Timer.getTime(tmr) > 1000 then
			if Timer.getTime(tmr) > 1600 then
				Timer.reset(tmr)
			end
		else
			Font.print(ttf,422, 352,"Press Start",white)
		end
		Font.print(ttf,870, 502,version,white)
		if Controls.check(pad, SCE_CTRL_START) and not Controls.check(oldpad, SCE_CTRL_START) then
			PlaySFX(step, NO_LOOP)
			menu_idx = 2
		end
	else
		if credits then
			DrawCredits()
		elseif load_screen then
			if #save_slots == 0 then
				load_screen = false
				idx_voices = 1
			end
			DrawSavegameMenu("LOAD GAME", false)
			if Controls.check(pad, SCE_CTRL_SQUARE) and not Controls.check(oldpad, SCE_CTRL_SQUARE) then
				PlaySFX(step, NO_LOOP)
				Graphics.termBlend()
				Screen.flip()
				if askPlayer("Do you really want to delete the savegame?", nil) then
					pad = SCE_CTRL_CROSS
					System.deleteFile(SAV_FOLDER .. "/" .. save_slots[save_idx].name)
					table.remove(save_slots, save_idx)
				end		
			elseif Controls.check(pad, SCE_CTRL_UP) and not Controls.check(oldpad, SCE_CTRL_UP) then
				PlaySFX(step, NO_LOOP)
				save_idx = save_idx - 1
				if save_idx < 1 then
					save_idx = #save_slots
				end
			elseif Controls.check(pad, SCE_CTRL_DOWN) and not Controls.check(oldpad, SCE_CTRL_DOWN) then
				PlaySFX(step, NO_LOOP)
				save_idx = save_idx + 1
				if save_idx > #save_slots then
					save_idx = 1
				end
			elseif Controls.check(pad, SCE_CTRL_CROSS) and not Controls.check(oldpad, SCE_CTRL_CROSS) then
				PlaySFX(step, NO_LOOP)
				Sound.pause(bgm)
				Sound.close(bgm)
				load_screen = false
				saved_playtime = save_slots[save_idx].playtime
				load_scrolls = save_slots[save_idx].scrolls
				load_x = save_slots[save_idx].x
				load_y = save_slots[save_idx].y
				load_angle = math.deg(arc2rad(save_slots[save_idx].angle))
				PrimmBuild(save_slots[save_idx].size, save_slots[save_idx].size, save_slots[save_idx].seed)
				local iter = 0
				load_marked_cells = {}
				while iter < string.len(save_slots[save_idx].marked) do
					local cell_x = str2int(string.sub(save_slots[save_idx].marked, iter+1, iter+4))
					local cell_y = str2int(string.sub(save_slots[save_idx].marked, iter+5, iter+8))
					local cell_idx = 1+cell_x+cell_y*wmap
					map[cell_idx] = M
					table.insert(load_marked_cells, {["x"] = cell_x,["y"] = cell_y})
					iter = iter + 8
				end
				dofile("app0:/files/game.lua")
				RayCast3D.loadMap(dmap, dmap_width, dmap_height, tile_size, wall_height)
				RayCast3D.spawnPlayer(96, 96, 90)
				bgm = Sound.open("app0:/files/menu.ogg")
				PlayMusic(bgm, LOOP)
				Timer.resume(tmr)
				state = 0
				config_update = true
			elseif Controls.check(pad, SCE_CTRL_CIRCLE) and not Controls.check(oldpad, SCE_CTRL_CIRCLE) then
				PlaySFX(step, NO_LOOP)
				load_screen = false
			end
		elseif seed_game then
			DrawSeedKeyboard()
			if seed_val ~= nil then
				seed_game = false
				Sound.pause(bgm)
				Sound.close(bgm)
				game_seed = tonumber(seed_val)
				PrimmBuild(tsizes[idx_sizes], tsizes[idx_sizes], game_seed)
				dofile("app0:/files/game.lua")
				RayCast3D.loadMap(dmap, dmap_width, dmap_height, tile_size, wall_height)
				RayCast3D.spawnPlayer(96, 96, 90)
				bgm = Sound.open("app0:/files/menu.ogg")
				PlayMusic(bgm, LOOP)
				Timer.resume(tmr)
				state = 0
				config_update = true
			end
		else
			if not easter_egg then
				if Controls.check(pad, konami_code[konami_idx]) and not Controls.check(oldpad, konami_code[konami_idx]) then
					konami_idx = konami_idx + 1
					Timer.reset(konami_tmr)
					if konami_idx > #konami_code then
						easter_egg = true
					end
				elseif Timer.getTime(konami_tmr) > 800 then
					konami_idx = 1
					Timer.reset(konami_tmr)
				end
				DrawMenu(menu_title)
				if Controls.check(pad, SCE_CTRL_CROSS) and not Controls.check(oldpad, SCE_CTRL_CROSS) then
					PlaySFX(step, NO_LOOP)
					if idx_voices == 1 then
						if menu_idx == 2 then
							menu_idx = 3
							menu_title = "NEW GAME"
							idx_voices = 1
						elseif menu_idx == 3 then
							idx_sizes = idx_sizes + 1
							if idx_sizes > #sizes then
								idx_sizes = 1
							end
							new_game_voices[1] = "Map Size : "..sizes[idx_sizes]
						elseif menu_idx == 4 then
							changeTextureQuality()
						elseif menu_idx == 5 then
							changeMusicValue()
						end
					elseif idx_voices == 2 then
						if menu_idx == 2 then
							load_screen = true
							save_idx = 1
						elseif menu_idx == 3 then
							Sound.pause(bgm)
							Sound.close(bgm)
							PrimmBuild(tsizes[idx_sizes], tsizes[idx_sizes], nil)
							dofile("app0:/files/game.lua")
							RayCast3D.loadMap(dmap, dmap_width, dmap_height, tile_size, wall_height)
							RayCast3D.spawnPlayer(96, 96, 90)
							bgm = Sound.open("app0:/files/menu.ogg")
							PlayMusic(bgm, LOOP)
							Timer.resume(tmr)
							state = 0
							config_update = true
						elseif menu_idx == 4 then
							changeShadingValue()
						elseif menu_idx == 5 then
							changeSoundValue()
						end
					elseif idx_voices == 3 then
						if menu_idx == 2 then
							menu_idx = 4
							idx_voices = 1
							menu_title = "GRAPHICS SETTINGS"
						elseif menu_idx == 3 then
							seed_game = true
							InitSeedKeyboard()
						elseif menu_idx == 4 then
							menu_idx = 2
							menu_title = "MAIN MENU"
							idx_voices = 3
							config_update = true
						elseif menu_idx == 5 then
							menu_idx = 2
							menu_title = "MAIN MENU"
							idx_voices = 4
							config_update = true
						end
					elseif idx_voices == 4 then
						if menu_idx == 2 then
							menu_idx = 5
							menu_title = "AUDIO SETTINGS"
							idx_voices = 1
						elseif menu_idx == 3 then
							menu_idx = 2
							menu_title = "MAIN MENU"
							idx_voices = 1
						end
					elseif idx_voices == 5 then
						if menu_idx == 2 then
							Timer.reset(tmr)
							credits = true
						end
					elseif idx_voices == 6 then
						if menu_idx == 2 then
							Sound.pause(bgm)
							Sound.close(bgm)
							Graphics.freeImage(W)
							Graphics.freeImage(logo)
							Graphics.freeImage(anim)
							Graphics.freeImage(skybox)
							Timer.destroy(tmr)
							Timer.destroy(konami_tmr)
							Timer.destroy(sec)
							Sound.close(step)
							Sound.close(walk)
							--Sound.term()
							Font.unload(ttf)
							System.exit()
						end
					end
				elseif Controls.check(pad, SCE_CTRL_UP) and not Controls.check(oldpad, SCE_CTRL_UP) then
					idx_voices = idx_voices - 1
					if menu_idx == 2 and idx_voices == 2 and #save_slots == 0 then
						idx_voices = 1
					end
					if idx_voices < 1 then
						idx_voices = #menus_voices[menu_idx]
					end
					PlaySFX(step, NO_LOOP)
				elseif Controls.check(pad, SCE_CTRL_DOWN) and not Controls.check(oldpad, SCE_CTRL_DOWN) then
					idx_voices = idx_voices + 1
					if menu_idx == 2 and idx_voices == 2 and #save_slots == 0 then
						idx_voices = 3
					end
					if idx_voices > #menus_voices[menu_idx] then
						idx_voices = 1
					end
					PlaySFX(step, NO_LOOP)
				elseif Controls.check(pad, SCE_CTRL_CIRCLE) and not Controls.check(oldpad, SCE_CTRL_CIRCLE) then
					PlaySFX(step, NO_LOOP)
					if seed_game then
						menu_idx = 3
					elseif menu_idx > 2 then
						config_update = true
						menu_idx = 2
					else
						menu_idx = 1
					end
				end
			else
				if Controls.check(pad, KEY_START) then
					easter_egg = false
					set_egg = true
					konami_idx = 1
				end
				cmove = false
				cx, cy = Controls.readLeftAnalog()
				cx2 = Controls.readRightAnalog()
				if cx2 > 180 then
					RayCast3D.rotateCamera(RIGHT, cam_speed)
				elseif cx2 < 64 then
					RayCast3D.rotateCamera(LEFT, cam_speed)
				end
				if cx > 180 then
					RayCast3D.movePlayer(RIGHT, pl_speed)
					if isSFX then
						Sound.resume(walk)
					end
				elseif cx < 64 then
					RayCast3D.movePlayer(LEFT, pl_speed)
					if isSFX then
						Sound.resume(walk)
					end
				end
				if cy < 64 then
					RayCast3D.movePlayer(FORWARD, pl_speed)
					cmove = true
					if isSFX then
						Sound.resume(walk)
					end
				elseif cy > 180 then
					RayCast3D.movePlayer(BACK, pl_speed)
					cmove = true
					if isSFX then
						Sound.resume(walk)
					end
				end
				if Controls.check(pad, SCE_CTRL_LEFT) then
					RayCast3D.rotateCamera(LEFT, cam_speed)
				end
				if Controls.check(pad, SCE_CTRL_RIGHT) then
					RayCast3D.rotateCamera(RIGHT, cam_speed)
				end
				if Controls.check(pad,SCE_CTRL_UP) then
					RayCast3D.movePlayer(FORWARD, pl_speed)
					if isSFX then
						Sound.resume(walk)
					end
				elseif Controls.check(pad,SCE_CTRL_DOWN) then
					RayCast3D.movePlayer(BACK, pl_speed)
					if isSFX then
						Sound.resume(walk)
					end
				elseif not cmove then
					Sound.pause(walk)
				end
			end
		end
	end
	
	Graphics.termBlend()
	oldpad = pad
	Screen.flip()
end